@@ -8,58 +8,62 @@
 This module contains the basic API for splinter drivers and elements.
 """
 
+from typing import Any, Optional, Type
+
 from splinter.meta import InheritedDocs
+from splinter.element_list import ElementList
+from splinter.cookie_manager import CookieManagerAPI
 
 
-class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
+class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):  # type: ignore
     """
     Basic driver API class.
     """
 
-    driver_name = ''
+    driver_name: str = ''
 
     @property
-    def title(self):
+    def title(self) -> str:
         """Title of current page."""
         raise NotImplementedError(
             "%s doesn't support access to the title." % self.driver_name
         )
 
-    def __enter__(self):
+    def __enter__(self) -> Any:
         """Context manager to use the browser safely."""
         raise NotImplementedError(
             "%s doesn't support use by 'with' statement." % self.driver_name
         )
 
-    def __exit__(self):
+    def __exit__(self) -> None:
         """Context manager to use the browser safely."""
         raise NotImplementedError(
             "%s doesn't support use by 'with' statement." % self.driver_name
         )
 
     @property
-    def html(self):
+    def html(self) -> str:
         """Source of current page."""
         raise NotImplementedError(
             "%s doesn't support access to the html." % self.driver_name
         )
 
     @property
-    def url(self):
+    def url(self) -> str:
         """URL of current page."""
         raise NotImplementedError(
             "%s doesn't support access to the url." % self.driver_name
         )
 
-    def visit(self, url):
-        """The browser will navigate to the given URL.
+    def visit(self, url: str) -> None:
+        """Use the browser to navigate to the given URL.
 
         Arguments:
             url (str): URL path.
         """
         raise NotImplementedError("%s doesn't visit any url." % self.driver_name)
 
-    def new_tab(self, url):
+    def new_tab(self, url: str) -> None:
         """The browser will navigate to the given URL in a new tab.
 
         Arguments:
@@ -67,7 +71,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
         """
         raise NotImplementedError("%s doesn't open a new tab." % self.driver_name)
 
-    def back(self):
+    def back(self) -> None:
         """The browser will navigate to the previous URL in the history.
 
         If there is no previous URL, this method does nothing.
@@ -76,7 +80,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             "%s doesn't support moving back in history." % self.driver_name
         )
 
-    def forward(self):
+    def forward(self) -> None:
         """The browser will navigate to the next URL in the history.
 
         If there is no URL to forward, this method does nothing.
@@ -85,27 +89,27 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             "%s doesn't support moving forward in history." % self.driver_name
         )
 
-    def reload(self):
+    def reload(self) -> None:
         """Revisits the current URL."""
         raise NotImplementedError(
             "%s doesn't support reloading the page." % self.driver_name
         )
 
-    def get_alert(self):
+    def get_alert(self) -> Any:
         """Change the context for working with alerts and prompts.
 
         For more details, check the :doc:`docs about iframes, alerts and prompts </iframes-and-alerts>`
         """
         raise NotImplementedError("%s doesn't support alerts." % self.driver_name)
 
-    def get_iframe(self, name):
+    def get_iframe(self, name: Any) -> Any:
         """Change the context for working with iframes.
 
         For more details, check the :doc:`docs about iframes, alerts and prompts </iframes-and-alerts>`
         """
         raise NotImplementedError("%s doesn't support frames." % self.driver_name)
 
-    def execute_script(self, script, *args):
+    def execute_script(self, script: str, *args: str) -> Any:
         """Execute a piece of JavaScript in the browser.
 
         Arguments:
@@ -119,7 +123,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             "%s doesn't support execution of arbitrary JavaScript." % self.driver_name
         )
 
-    def evaluate_script(self, script, *args):
+    def evaluate_script(self, script: str, *args: str) -> Any:
         """
         Similar to :meth:`execute_script <DriverAPI.execute_script>` method.
 
@@ -136,7 +140,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             "%s doesn't support evaluation of arbitrary JavaScript." % self.driver_name
         )
 
-    def find_by_css(self, css_selector):
+    def find_by_css(self, css_selector: str) -> ElementList:
         """
         Return an instance of :class:`ElementList <splinter.element_list.ElementList>`,
         using a CSS selector to query the current page content.
@@ -149,7 +153,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             "%s doesn't support finding elements by css selector." % self.driver_name
         )
 
-    def find_by_xpath(self, xpath):
+    def find_by_xpath(self, xpath: str) -> ElementList:
         """
         Return an instance of :class:`ElementList <splinter.element_list.ElementList>`,
         using a xpath selector to query the current page content.
@@ -162,7 +166,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             "%s doesn't support finding elements by xpath selector." % self.driver_name
         )
 
-    def find_by_name(self, name):
+    def find_by_name(self, name: str) -> ElementList:
         """Find elements on the current page by their name.
 
         Return an instance of :class:`ElementList <splinter.element_list.ElementList>`.
@@ -175,7 +179,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             "%s doesn't support finding elements by name." % self.driver_name
         )
 
-    def find_by_id(self, id):  # NOQA: A002
+    def find_by_id(self, id: str) -> ElementList:  # NOQA: A002
         """Find an element on the current page by its id.
 
         Even when only one element is find, this method returns an instance of
@@ -188,7 +192,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             "%s doesn't support finding elements by id." % self.driver_name
         )
 
-    def find_by_value(self, value):
+    def find_by_value(self, value: str) -> ElementList:
         """Find elements on the current page by their value.
 
         Returns an instance of :class:`ElementList <splinter.element_list.ElementList>`
@@ -200,7 +204,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             "%s doesn't support finding elements by value." % self.driver_name
         )
 
-    def find_by_text(self, text):
+    def find_by_text(self, text: str) -> ElementList:
         """Find elements on the current page by their text.
 
         Returns an instance of :class:`ElementList <splinter.element_list.ElementList>`
@@ -212,7 +216,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             "%s doesn't support finding elements by text." % self.driver_name
         )
 
-    def find_by_tag(self, tag):
+    def find_by_tag(self, tag: str) -> ElementList:
         """Find all elements of a given tag in current page.
 
         Returns an instance of :class:`ElementList <splinter.element_list.ElementList>`
@@ -224,7 +228,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             "%s doesn't support finding elements by tag." % self.driver_name
         )
 
-    def find_option_by_value(self, value):
+    def find_option_by_value(self, value: str) -> ElementList:
         """
         Find ``<option>`` elements by their value.
 
@@ -237,7 +241,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             "%s doesn't support finding options by value." % self.driver_name
         )
 
-    def find_option_by_text(self, text):
+    def find_option_by_text(self, text: str) -> ElementList:
         """
         Finds ``<option>`` elements by their text.
 
@@ -250,7 +254,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             "%s doesn't support finding options by text." % self.driver_name
         )
 
-    def is_text_present(self, text, wait_time=None):
+    def is_text_present(self, text: str, wait_time: Optional[int] = None) -> bool:
         """Check if a piece of text is on the page.
 
         Arguments:
@@ -265,7 +269,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             % self.driver_name
         )
 
-    def type(self, name, value, slowly=False):  # NOQA: A003
+    def type(self, name: str, value: str, slowly: bool = False) -> str:  # NOQA: A003
         """Type a value into an element.
 
         It's useful to test javascript events like keyPress, keyUp, keyDown, etc.
@@ -279,7 +283,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             "%s doesn't support typing on fields by name." % self.driver_name
         )
 
-    def fill(self, name, value):
+    def fill(self, name: str, value: str) -> None:
         """Fill the field identified by ``name`` with the content specified by ``value``.
 
         Arguments:
@@ -290,7 +294,12 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             "%s doesn't support filling fields by name." % self.driver_name
         )
 
-    def fill_form(self, field_values, form_id=None, name=None):
+    def fill_form(
+        self,
+        field_values,
+        form_id: Optional[str] = None,
+        name: Optional[str] = None,
+    ) -> None:
         """
         Fill the fields identified by ``name`` with the content specified by ``value`` in a dict.
 
@@ -309,7 +318,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             "%s doesn't support filling forms with a dict." % self.driver_name
         )
 
-    def choose(self, name, value):
+    def choose(self, name: str, value: str) -> None:
         """Choose a value in a radio buttons group.
 
         Arguments:
@@ -328,7 +337,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             "%s doesn't support choosing options." % self.driver_name
         )
 
-    def check(self, name):
+    def check(self, name: str) -> None:
         """Check a checkbox by its name.
 
         Arguments:
@@ -346,7 +355,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             "%s doesn't support checking elements." % self.driver_name
         )
 
-    def uncheck(self, name):
+    def uncheck(self, name: str) -> None:
         """Uncheck a checkbox by its name.
 
         Arguments:
@@ -364,7 +373,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             "%s doesn't support unchecking elements." % self.driver_name
         )
 
-    def select(self, name, value):
+    def select(self, name: str, value: str) -> None:
         """
         Select an ``<option>`` element in an ``<select>`` element using the ``name`` of the ``<select>`` and
         the ``value`` of the ``<option>``.
@@ -382,11 +391,11 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             % self.driver_name
         )
 
-    def quit(self):  # NOQA: A003
+    def quit(self) -> None:  # NOQA: A003
         """Quit the browser, closing its windows (if it has one)."""
         raise NotImplementedError("%s doesn't support quit" % self.driver_name)
 
-    def is_element_present_by_css(self, css_selector, wait_time=None):
+    def is_element_present_by_css(self, css_selector: str, wait_time: Optional[int] = None) -> bool:
         """Verify if an element is present in the current page.
 
         Arguments:
@@ -401,7 +410,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             % self.driver_name
         )
 
-    def is_element_not_present_by_css(self, css_selector, wait_time=None):
+    def is_element_not_present_by_css(self, css_selector: str, wait_time: Optional[int] = None) -> bool:
         """Verify if an element is not present in the current page.
 
         Arguments:
@@ -416,7 +425,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             % self.driver_name
         )
 
-    def is_element_present_by_xpath(self, xpath, wait_time=None):
+    def is_element_present_by_xpath(self, xpath: str, wait_time: Optional[int] = None) -> bool:
         """Verify if an element is present in the current page.
 
         Arguments:
@@ -431,7 +440,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             % self.driver_name
         )
 
-    def is_element_not_present_by_xpath(self, xpath, wait_time=None):
+    def is_element_not_present_by_xpath(self, xpath: str, wait_time: Optional[int] = None) -> bool:
         """Verify if an element is not present in the current page.
 
         Arguments:
@@ -446,7 +455,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             % self.driver_name
         )
 
-    def is_element_present_by_tag(self, tag, wait_time=None):
+    def is_element_present_by_tag(self, tag: str, wait_time: Optional[int] = None) -> bool:
         """Verify if an element is present in the current page.
 
         Arguments:
@@ -461,7 +470,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             % self.driver_name
         )
 
-    def is_element_not_present_by_tag(self, tag, wait_time=None):
+    def is_element_not_present_by_tag(self, tag: str, wait_time: Optional[int] = None) -> bool:
         """Verify if an element is not present in the current page.
 
         Arguments:
@@ -476,7 +485,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             % self.driver_name
         )
 
-    def is_element_present_by_name(self, name, wait_time=None):
+    def is_element_present_by_name(self, name: str, wait_time: Optional[int] = None) -> bool:
         """Verify if an element is present in the current page.
 
         Arguments:
@@ -491,7 +500,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             % self.driver_name
         )
 
-    def is_element_not_present_by_name(self, name, wait_time=None):
+    def is_element_not_present_by_name(self, name: str, wait_time: Optional[int] = None) -> bool:
         """Verify if an element is not present in the current page.
 
         Arguments:
@@ -506,7 +515,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             % self.driver_name
         )
 
-    def is_element_present_by_value(self, value, wait_time=None):
+    def is_element_present_by_value(self, value: str, wait_time: Optional[int] = None) -> bool:
         """Verify if an element is present in the current page.
 
         Arguments:
@@ -521,7 +530,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             % self.driver_name
         )
 
-    def is_element_not_present_by_value(self, value, wait_time=None):
+    def is_element_not_present_by_value(self, value: str, wait_time: Optional[int] = None) -> bool:
         """Verify if an element is not present in the current page.
 
         Arguments:
@@ -536,7 +545,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             % self.driver_name
         )
 
-    def is_element_present_by_text(self, text, wait_time=None):
+    def is_element_present_by_text(self, text: str, wait_time: Optional[int] = None) -> bool:
         """Verify if an element is present in the current page.
 
         Arguments:
@@ -551,7 +560,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             % self.driver_name
         )
 
-    def is_element_not_present_by_text(self, text, wait_time=None):
+    def is_element_not_present_by_text(self, text: str, wait_time: Optional[int] = None) -> bool:
         """Verify if an element is not present in the current page.
 
         Arguments:
@@ -566,7 +575,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             % self.driver_name
         )
 
-    def is_element_present_by_id(self, id, wait_time=None):  # NOQA: A002
+    def is_element_present_by_id(self, id: str, wait_time: Optional[int] = None) -> bool:  # NOQA: A002
         """Verify if an element is present in the current page.
 
         Arguments:
@@ -581,7 +590,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             % self.driver_name
         )
 
-    def is_element_not_present_by_id(self, id, wait_time=None):  # NOQA: A002
+    def is_element_not_present_by_id(self, id: str, wait_time: Optional[int] = None) -> bool:  # NOQA: A002
         """Verify if an element is not present in the current page.
 
         Arguments:
@@ -596,7 +605,13 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
             % self.driver_name
         )
 
-    def screenshot(self, name=None, suffix=None, full=False, unique_file=True):
+    def screenshot(
+        self,
+        name: Optional[str] = None,
+        suffix: Optional[str] = None,
+        full: bool = False,
+        unique_file: bool = True,
+    ) -> str:
         """Take a screenshot of the current page and save it locally.
 
         Arguments:
@@ -640,7 +655,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
         )
 
     @property
-    def cookies(self):
+    def cookies(self) -> Type[CookieManagerAPI]:
         """
         A :class:`CookieManager <splinter.cookie_manager.CookieManagerAPI>` instance.
 
@@ -651,7 +666,7 @@ class DriverAPI(InheritedDocs("_DriverAPI", (object,), {})):
         )
 
 
-class ElementAPI(InheritedDocs("_ElementAPI", (object,), {})):
+class ElementAPI(InheritedDocs("_ElementAPI", (object,), {})):  # type: ignore
     """
     Basic element API class.
 
@@ -665,10 +680,10 @@ class ElementAPI(InheritedDocs("_ElementAPI", (object,), {})):
     You can also interact with the instance using the methods and properties listed below.
     """
 
-    def _get_value(self):
+    def _get_value(self) -> str:
         raise NotImplementedError
 
-    def _set_value(self, value):
+    def _set_value(self, value: str) -> None:
         raise NotImplementedError
 
     #: Value of the element, usually a form element
@@ -680,19 +695,15 @@ class ElementAPI(InheritedDocs("_ElementAPI", (object,), {})):
         raise NotImplementedError
 
     @property
-    def text(self):
-        """
-        String of all of the text within the element.  HTML tags are stripped.
-        """
+    def text(self) -> str:
+        """All of the text within the element.  HTML tags are stripped."""
         raise NotImplementedError
 
-    def click(self):
-        """
-        Click in the element.
-        """
+    def click(self) -> None:
+        """Click the element."""
         raise NotImplementedError
 
-    def check(self):
+    def check(self) -> None:
         """
         Check the element, if it's "checkable" (e.g.: a checkbox).
 
@@ -701,7 +712,7 @@ class ElementAPI(InheritedDocs("_ElementAPI", (object,), {})):
         """
         raise NotImplementedError
 
-    def uncheck(self):
+    def uncheck(self) -> None:
         """
         Uncheck the element, if it's "checkable" (e.g.: a checkbox).
 
@@ -711,9 +722,8 @@ class ElementAPI(InheritedDocs("_ElementAPI", (object,), {})):
         raise NotImplementedError
 
     @property
-    def checked(self):
-        """
-        Boolean property that says if the element is checked or not.
+    def checked(self) -> bool:
+        """Get the checked status of the element.
 
         Example:
 
@@ -725,49 +735,42 @@ class ElementAPI(InheritedDocs("_ElementAPI", (object,), {})):
         raise NotImplementedError
 
     @property
-    def visible(self):
-        """
-        Boolean property that says if the element is visible or hidden in the current page.
-        """
+    def visible(self) -> bool:
+        """Get the visibility status of the element."""
         raise NotImplementedError
 
-    def has_class(self, class_name):
+    def has_class(self, class_name: str) -> bool:
         """
         Indicates whether the element has the given class.
         """
         raise NotImplementedError
 
-    def mouse_over(self):
-        """
-        Put the mouse over the element.
-        """
+    def mouse_over(self) -> None:
+        """Move the mouse over the element."""
         raise NotImplementedError
 
-    def mouse_out(self):
-        """
-        Move the mouse away from the element.
-        """
+    def mouse_out(self) -> None:
+        """Move the mouse away from the element."""
         raise NotImplementedError
 
-    def clear(self):
+    def clear(self) -> None:
         """Reset the field value."""
         raise NotImplementedError
 
-    def fill(self, value):
-        """
-        Fill the field with the content specified by ``value``.
+    def fill(self, value: str) -> None:
+        """Fill the element with text content.
+
+        Arguments:
+            value (str): Value to enter into the element.
         """
         raise NotImplementedError
 
-    def type(self, value, slowly=False):  # NOQA: A003
-        """
-        Type the ``value`` in the field.
-
-        It's useful to test javascript events like keyPress, keyUp, keyDown, etc.
+    def type(self, value: str, slowly: bool = False) -> str:  # NOQA: A003
+        """Type the ``value`` in the element.
 
         If ``slowly`` is True, this function returns an iterator which will type one character per iteration.
 
-        Also supports selenium.
+        It's useful to test javascript events like keyPress, keyUp, keyDown, etc.
 
         Example:
 
@@ -777,7 +780,7 @@ class ElementAPI(InheritedDocs("_ElementAPI", (object,), {})):
         """
         raise NotImplementedError
 
-    def select(self, value, slowly=False):
+    def select(self, value: str, slowly: bool = False) -> None:
         """
         Select an ``<option>`` element in the element using the ``value`` of the ``<option>``.
 
@@ -787,9 +790,15 @@ class ElementAPI(InheritedDocs("_ElementAPI", (object,), {})):
         """
         raise NotImplementedError
 
-    def screenshot(self, name=None, suffix=None, full=False, unique_file=True):
-        """Take screenshot of the element."""
+    def screenshot(
+        self,
+        name: Optional[str] = None,
+        suffix: Optional[str] = None,
+        full: bool = False,
+        unique_file: bool = True,
+    ) -> str:
+        """Take a screenshot of the element."""
         raise NotImplementedError
 
-    def __getitem__(self, attribute):
+    def __getitem__(self, attribute: str) -> Any:
         raise NotImplementedError
