@@ -6,10 +6,8 @@
 
 import re
 import time
-import sys
 
-import six
-from six.moves.urllib import parse
+from urllib import parse
 
 import lxml.etree
 import lxml.html
@@ -30,6 +28,7 @@ class LxmlDriver(ElementPresentMixIn, DriverAPI):
         self.wait_time = wait_time
         self._history = []
         self._last_urls = []
+        self._last_url_index = -1  # Empty
         self._forms = {}
 
         self.links = FindLinks(self)
@@ -52,10 +51,10 @@ class LxmlDriver(ElementPresentMixIn, DriverAPI):
         data = {}
 
         for key in form.inputs.keys():
-            input = form.inputs[key]
-            if getattr(input, "type", "") == "submit":
+            form_input = form.inputs[key]
+            if getattr(form_input, "type", "") == "submit":
                 try:
-                    form.remove(input)
+                    form.remove(form_input)
                 # Issue 595: throws ValueError: Element not child of this node
                 except ValueError:
                     pass
@@ -70,8 +69,8 @@ class LxmlDriver(ElementPresentMixIn, DriverAPI):
                 data[k] = v
 
         for key in form.inputs.keys():
-            input = form.inputs[key]
-            if getattr(input, "type", "") == "file" and key in data:
+            form_input = form.inputs[key]
+            if getattr(form_input, "type", "") == "file" and key in data:
                 data[key] = open(data[key], "rb")
 
         return data
@@ -96,19 +95,19 @@ class LxmlDriver(ElementPresentMixIn, DriverAPI):
         )
 
     def back(self):
-        self._last_urls.insert(0, self.url)
-        self.visit(self._last_urls[1])
+        if self._last_url_index >= 1:
+            self._last_url_index -= 1
+            self._do_method("get", self._last_urls[self._last_url_index], record_url=False)
 
     def forward(self):
-        try:
-            self.visit(self._last_urls.pop())
-        except IndexError:
-            pass
+        if (self._last_url_index >= 0) and (self._last_url_index < len(self._last_urls) - 1):
+            self._last_url_index += 1
+            self._do_method("get", self._last_urls[self._last_url_index], record_url=False)
 
     def reload(self):
         self.visit(self._url)
 
-    def quit(self):
+    def quit(self):  # NOQA: A003
         pass
 
     @property
@@ -224,20 +223,6 @@ class LxmlDriver(ElementPresentMixIn, DriverAPI):
             query=query,
         )
 
-    def find_link_by_text(self, text):
-        return self._find_links_by_xpath("//a[text()='%s']" % text)
-
-    def find_link_by_href(self, href):
-        return self._find_links_by_xpath("//a[@href='%s']" % href)
-
-    def find_link_by_partial_href(self, partial_href):
-        return self._find_links_by_xpath("//a[contains(@href, '%s')]" % partial_href)
-
-    def find_link_by_partial_text(self, partial_text):
-        return self._find_links_by_xpath(
-            "//a[contains(normalize-space(.), '%s')]" % partial_text
-        )
-
     def fill(self, name, value):
         self.find_by_name(name=name).first.fill(value)
 
@@ -378,7 +363,7 @@ class LxmlElement(ElementAPI):
         xpath_str = '.{}'.format(_concat_xpath_from_str(text))
         return self.find_by_xpath(xpath_str)
 
-    def find_by_id(self, id):
+    def find_by_id(self, id):  # NOQA: A002
         elements = self._element.cssselect("#%s" % id)
         return ElementList([self.__class__(element, self) for element in elements])
 
@@ -449,11 +434,6 @@ class LxmlControlElement(LxmlElement):
 
     def fill(self, value):
         parent_form = self._get_parent_form()
-
-        if not sys.version_info[0] > 2:
-            if not isinstance(value, six.text_type):
-                value = value.decode("utf-8")
-
         parent_form.fields[self["name"]] = value
 
     def select(self, value):
