@@ -7,11 +7,9 @@
 import io
 import os
 import re
-import sys
 import tempfile
 import time
 from contextlib import contextmanager
-import warnings
 
 from selenium.webdriver.common.alert import Alert
 from selenium.common.exceptions import (
@@ -23,33 +21,24 @@ from selenium.common.exceptions import (
     MoveTargetOutOfBoundsException,
 )
 from selenium.webdriver.common.action_chains import ActionChains
+from selenium.webdriver.common.by import By
 from selenium.webdriver.support import expected_conditions as EC  # NOQA: N812
 from selenium.webdriver.support.ui import WebDriverWait
 
-from six import BytesIO
-
 from splinter.driver import DriverAPI, ElementAPI
 from splinter.driver.find_links import FindLinks
 from splinter.driver.xpath_utils import _concat_xpath_from_str
 from splinter.element_list import ElementList
-
+from splinter.driver.webdriver.cookie_manager import CookieManager
 from splinter.exceptions import ElementDoesNotExist
 
 
-if sys.version_info[0] > 2:
-    _meth_func = "__func__"
-    _func_name = "__name__"
-else:
-    _meth_func = "im_func"
-    _func_name = "func_name"
-
-
 # Patch contextmanager onto Selenium's Alert
 def alert_enter(self):
     return self
 
 
-def alert_exit(self, type, value, traceback):
+def alert_exit(self, type, value, traceback):  # NOQA: A002
     pass
 
 
@@ -67,12 +56,12 @@ class switch_window:  # NOQA: N801
         self.current_window_handle = self.browser.driver.current_window_handle
         self.browser.driver.switch_to.window(self.window_handle)
 
-    def __exit__(self, type, value, traceback):
+    def __exit__(self, type, value, traceback):  # NOQA: A002
         if self.current_window_handle in self.browser.driver.window_handles:
             self.browser.driver.switch_to.window(self.current_window_handle)
 
 
-class Window(object):
+class Window:
     """ A class representing a browser window """
 
     def __init__(self, browser, name):
@@ -103,8 +92,8 @@ class Window(object):
         prev_handle = self._browser.driver.window_handles[prev_index]
         return Window(self._browser, prev_handle)
 
-    @property
-    def next(self):
+    @property  # NOQA: A003
+    def next(self):  # NOQA: A003
         """ Return the next window """
         next_index = (self.index + 1) % len(self._browser.driver.window_handles)
         next_handle = self._browser.driver.window_handles[next_index]
@@ -175,7 +164,7 @@ class Window(object):
         return "<Window %s: %s>" % (self.name, self.url)
 
 
-class Windows(object):
+class Windows:
 
     """ A class representing all open browser windows """
 
@@ -217,22 +206,24 @@ class Windows(object):
         )
 
 
-def _find(self, finder, selector):
+def _find(self, finder, finder_kwargs=None):
     """Search for elements. Returns a list of results.
 
     Arguments:
         finder: The function to use for the element search.
-        selector: The search query.
+        finder_kwargs: Keyword Arguments for the finder function.
 
     Returns:
         list
 
     """
+    finder_kwargs = finder_kwargs or {}
+
     elements = None
     elem_list = []
 
     try:
-        elements = finder(selector)
+        elements = finder(**finder_kwargs)
         if not isinstance(elements, list):
             elements = [elements]
 
@@ -250,7 +241,14 @@ def _find(self, finder, selector):
     return elem_list
 
 
-def find_by(self, finder, selector, original_find=None, original_query=None, wait_time=None):
+def find_by(
+    self,
+    finder,
+    finder_kwargs=None,
+    original_find: str = None,
+    original_query: str = None,
+    wait_time: int = None,
+):
     """Wrapper for finding elements.
 
     Must be attached to a class.
@@ -261,19 +259,18 @@ def find_by(self, finder, selector, original_find=None, original_query=None, wai
     """
     elem_list = []
 
-    func_name = getattr(getattr(finder, _meth_func), _func_name)
-    find_by = original_find or func_name[func_name.rfind("_by_") + 4:]
-    query = original_query or selector
+    find_by = original_find or finder_kwargs['by']
+    query = original_query or finder_kwargs.get('value')
 
     # Zero second wait time means only check once
     if wait_time == 0:
-        elem_list = _find(self, finder, selector)
+        elem_list = _find(self, finder, finder_kwargs)
     else:
         wait_time = wait_time or self.wait_time
         end_time = time.time() + wait_time
 
         while time.time() < end_time:
-            elem_list = _find(self, finder, selector)
+            elem_list = _find(self, finder, finder_kwargs)
 
             if elem_list:
                 break
@@ -285,11 +282,17 @@ class BaseWebDriver(DriverAPI):
     driver = None
     find_by = find_by
 
-    def __init__(self, wait_time=2):
+    def __init__(self, driver=None, wait_time=2):
         self.wait_time = wait_time
 
         self.links = FindLinks(self)
 
+        self.driver = driver
+
+        self.element_class = WebDriverElement
+
+        self._cookie_manager = CookieManager(self.driver)
+
     def __enter__(self):
         return self
 
@@ -415,10 +418,10 @@ class BaseWebDriver(DriverAPI):
     def is_element_not_present_by_text(self, text, wait_time=None):
         return self.is_element_not_present(self.find_by_text, text, wait_time)
 
-    def is_element_present_by_id(self, id, wait_time=None):
+    def is_element_present_by_id(self, id, wait_time=None):  # NOQA: A002
         return self.is_element_present(self.find_by_id, id, wait_time)
 
-    def is_element_not_present_by_id(self, id, wait_time=None):
+    def is_element_not_present_by_id(self, id, wait_time=None):  # NOQA: A002
         return self.is_element_not_present(self.find_by_id, id, wait_time)
 
     def get_alert(self, wait_time=None):
@@ -430,25 +433,24 @@ class BaseWebDriver(DriverAPI):
         except TimeoutException:
             return None
 
+    def _is_text_present(self, text):
+        try:
+            self.find_by_tag("body").text.index(text)
+            return True
+        except (NoSuchElementException, StaleElementReferenceException, ValueError):
+            # NoSuchElementException will be thrown if the body tag isn't present
+            # Can occur if the page isn't fully loaded yet.
+            # StaleElementReferenceException will be thrown if the page changes quickly
+            pass
+        return False
+
     def is_text_present(self, text, wait_time=None):
         wait_time = wait_time or self.wait_time
         end_time = time.time() + wait_time
 
         while time.time() < end_time:
-            try:
-                self.driver.find_element_by_tag_name("body").text.index(text)
+            if self._is_text_present(text):
                 return True
-            except ValueError:
-                pass
-            except NoSuchElementException:
-                # This exception will be thrown if the body tag isn't present
-                # This has occasionally been observed. Assume that the
-                # page isn't fully loaded yet
-                pass
-            except StaleElementReferenceException:
-                # This exception is sometimes thrown if the page changes
-                # quickly
-                pass
         return False
 
     def is_text_not_present(self, text, wait_time=None):
@@ -456,19 +458,8 @@ class BaseWebDriver(DriverAPI):
         end_time = time.time() + wait_time
 
         while time.time() < end_time:
-            try:
-                self.driver.find_element_by_tag_name("body").text.index(text)
-            except ValueError:
+            if not self._is_text_present(text):
                 return True
-            except NoSuchElementException:
-                # This exception will be thrown if the body tag isn't present
-                # This has occasionally been observed. Assume that the
-                # page isn't fully loaded yet
-                pass
-            except StaleElementReferenceException:
-                # This exception is sometimes thrown if the page changes
-                # quickly
-                pass
         return False
 
     @contextmanager
@@ -499,53 +490,20 @@ class BaseWebDriver(DriverAPI):
             original_query=text,
         )
 
-    def find_link_by_href(self, href):
-        warnings.warn(
-            'browser.find_link_by_href is deprecated.'
-            ' Use browser.links.find_by_href instead.',
-            FutureWarning,
-        )
-        return self.links.find_by_href(href)
-
-    def find_link_by_partial_href(self, partial_href):
-        warnings.warn(
-            'browser.find_link_by_partial_href is deprecated.'
-            ' Use browser.links.find_by_partial_href instead.',
-            FutureWarning,
-        )
-        return self.links.find_by_partial_href(partial_href)
-
-    def find_link_by_partial_text(self, partial_text):
-        warnings.warn(
-            'browser.find_link_by_partial_text is deprecated.'
-            ' Use browser.links.find_by_partial_text instead.',
-            FutureWarning,
-        )
-        return self.links.find_by_partial_text(partial_text)
-
-    def find_link_by_text(self, text):
-        warnings.warn(
-            'browser.find_link_by_text is deprecated.'
-            ' Use browser.links.find_by_text instead.',
-            FutureWarning,
-        )
-        return self.links.find_by_text(text)
-
     def find_by_css(self, css_selector, wait_time=None):
         return self.find_by(
-            self.driver.find_elements_by_css_selector,
-            css_selector,
+            self.driver.find_elements,
+            finder_kwargs={'by': By.CSS_SELECTOR, 'value': css_selector},
             original_find="css",
             original_query=css_selector,
             wait_time=wait_time,
         )
 
-    def find_by_xpath(self, xpath, original_find=None, original_query=None, wait_time=None):
-        original_find = original_find or "xpath"
+    def find_by_xpath(self, xpath, original_find="xpath", original_query=None, wait_time=None):
         original_query = original_query or xpath
         return self.find_by(
-            self.driver.find_elements_by_xpath,
-            xpath,
+            self.driver.find_elements,
+            finder_kwargs={'by': By.XPATH, 'value': xpath},
             original_find=original_find,
             original_query=original_query,
             wait_time=wait_time,
@@ -553,15 +511,17 @@ class BaseWebDriver(DriverAPI):
 
     def find_by_name(self, name, wait_time=None):
         return self.find_by(
-            self.driver.find_elements_by_name,
-            name,
+            self.driver.find_elements,
+            finder_kwargs={'by': By.NAME, 'value': name},
+            original_find='name',
             wait_time=wait_time,
         )
 
     def find_by_tag(self, tag, wait_time=None):
         return self.find_by(
-            self.driver.find_elements_by_tag_name,
-            tag,
+            self.driver.find_elements,
+            finder_kwargs={'by': By.TAG_NAME, 'value': tag},
+            original_find='tag_name',
             wait_time=wait_time,
         )
 
@@ -585,10 +545,11 @@ class BaseWebDriver(DriverAPI):
             wait_time=wait_time,
         )
 
-    def find_by_id(self, id, wait_time=None):
+    def find_by_id(self, id, wait_time=None):  # NOQA: A002
         return self.find_by(
-            self.driver.find_element_by_id,
-            id,
+            self.driver.find_element,
+            finder_kwargs={'by': By.ID, 'value': id},
+            original_find='id',
             wait_time=wait_time,
         )
 
@@ -635,7 +596,7 @@ class BaseWebDriver(DriverAPI):
                 if not ignore_missing:
                     raise ElementDoesNotExist(e)
 
-    def type(self, name, value, slowly=False):
+    def type(self, name, value, slowly=False):  # NOQA: A003
         element = self.find_by_name(name).first._element
         if slowly:
             return TypeIterator(element, value)
@@ -654,13 +615,13 @@ class BaseWebDriver(DriverAPI):
     def uncheck(self, name):
         self.find_by_name(name).first.uncheck()
 
-    def screenshot(self, name="", suffix=".png", full=False):
+    def screenshot(self, name="", suffix=".png", full=False, unique_file=True):
+        filename = '{}{}'.format(name, suffix)
 
-        name = name or ""
-
-        (fd, filename) = tempfile.mkstemp(prefix=name, suffix=suffix)
-        # don't hold the file
-        os.close(fd)
+        if unique_file:
+            (fd, filename) = tempfile.mkstemp(prefix=name, suffix=suffix)
+            # Don't hold the file
+            os.close(fd)
 
         if full:
             ori_window_size = self.driver.get_window_size()
@@ -673,6 +634,19 @@ class BaseWebDriver(DriverAPI):
 
         return filename
 
+    def html_snapshot(self, name="", suffix=".html", encoding='utf-8', unique_file=True):
+        filename = '{}{}'.format(name, suffix)
+
+        if unique_file:
+            (fd, filename) = tempfile.mkstemp(prefix=name, suffix=suffix)
+            # Don't hold the file
+            os.close(fd)
+
+        with io.open(filename, 'w', encoding=encoding) as f:
+            f.write(self.html)
+
+        return filename
+
     def select(self, name, value):
         self.find_by_xpath(
             '//select[@name="%s"]//option[@value="%s"]' % (name, value)
@@ -683,7 +657,7 @@ class BaseWebDriver(DriverAPI):
             '//select[@name="%s"]/option[text()="%s"]' % (name, text)
         ).first._element.click()
 
-    def quit(self):
+    def quit(self):  # NOQA: A003
         try:
             self.driver.quit()
         except WebDriverException:
@@ -699,19 +673,6 @@ class BaseWebDriver(DriverAPI):
         height = size.get('height')
         self.driver.set_window_size(width, height)
 
-    def html_snapshot(self, name="", suffix=".html", encoding='utf-8'):
-        """Write the current html to a file."""
-        name = name or ""
-
-        (fd, filename) = tempfile.mkstemp(prefix=name, suffix=suffix)
-        # Don't hold the file
-        os.close(fd)
-
-        with io.open(filename, 'w', encoding=encoding) as f:
-            f.write(self.html)
-
-        return filename
-
     @property
     def cookies(self):
         return self._cookie_manager
@@ -732,6 +693,37 @@ class TypeIterator(object):
             yield key
 
 
+class ShadowRootElement(ElementAPI):
+    find_by = find_by
+
+    def __init__(self, element, parent):
+        self._element = element
+        self.parent = parent
+
+        self.driver = self.parent.driver
+        self.wait_time = self.parent.wait_time
+        self.element_class = self.parent.element_class
+
+    def _find(self, by: By, selector, wait_time=None):
+        return self.find_by(
+            self._element.find_elements,
+            finder_kwargs={'by': by, 'value': selector},
+            wait_time=wait_time,
+        )
+
+    def find_by_css(self, selector, wait_time=None):
+        return self._find(By.CSS_SELECTOR, selector, wait_time)
+
+    def find_by_id(self, selector, wait_time=None):
+        return self._find(By.ID, selector, wait_time)
+
+    def find_by_name(self, selector, wait_time=None):
+        return self._find(By.NAME, selector, wait_time)
+
+    def find_by_tag(self, selector, wait_time=None):
+        return self._find(By.TAG_NAME, selector, wait_time)
+
+
 class WebDriverElement(ElementAPI):
     find_by = find_by
 
@@ -755,6 +747,9 @@ class WebDriverElement(ElementAPI):
 
     value = property(_get_value, _set_value)
 
+    def __getitem__(self, attr):
+        return self._element.get_attribute(attr)
+
     @property
     def text(self):
         return self._element.text
@@ -797,7 +792,7 @@ class WebDriverElement(ElementAPI):
     def select_by_text(self, text):
         self.select(text=text)
 
-    def type(self, value, slowly=False):
+    def type(self, value, slowly=False):  # NOQA: A003
         if slowly:
             return TypeIterator(self._element, value)
 
@@ -852,16 +847,16 @@ class WebDriverElement(ElementAPI):
 
     def find_by_css(self, selector, wait_time=None):
         return self.find_by(
-            self._element.find_elements_by_css_selector,
-            selector,
+            self._element.find_elements,
+            finder_kwargs={'by': By.CSS_SELECTOR, 'value': selector},
             original_find="css",
             wait_time=wait_time,
         )
 
     def find_by_xpath(self, selector, wait_time=None, original_find="xpath", original_query=None):
         return self.find_by(
-            self._element.find_elements_by_xpath,
-            selector,
+            self._element.find_elements,
+            finder_kwargs={'by': By.XPATH, 'value': selector},
             original_find=original_find,
             original_query=original_query,
             wait_time=wait_time,
@@ -869,25 +864,26 @@ class WebDriverElement(ElementAPI):
 
     def find_by_name(self, selector, wait_time=None):
         return self.find_by(
-            self._element.find_elements_by_name,
-            selector,
+            self._element.find_elements,
+            finder_kwargs={'by': By.NAME, 'value': selector},
             original_find="name",
             wait_time=wait_time,
         )
 
     def find_by_tag(self, selector, wait_time=None):
         return self.find_by(
-            self._element.find_elements_by_tag_name,
-            selector,
+            self._element.find_elements,
+            finder_kwargs={'by': By.TAG_NAME, 'value': selector},
             original_find="tag",
             wait_time=wait_time,
         )
 
     def find_by_value(self, value, wait_time=None):
         selector = '[value="{}"]'.format(value)
+
         return self.find_by(
-            self._element.find_elements_by_css_selector,
-            selector,
+            self._element.find_elements,
+            finder_kwargs={'by': By.CSS_SELECTOR, 'value': selector},
             original_find="value",
             original_query=value,
             wait_time=wait_time,
@@ -896,9 +892,10 @@ class WebDriverElement(ElementAPI):
     def find_by_text(self, text, wait_time=None):
         # Add a period to the xpath to search only inside the parent.
         xpath_str = '.{}'.format(_concat_xpath_from_str(text))
+
         return self.find_by(
-            self._element.find_elements_by_xpath,
-            xpath_str,
+            self._element.find_elements,
+            finder_kwargs={'by': By.XPATH, 'value': xpath_str},
             original_find="text",
             original_query=text,
             wait_time=wait_time,
@@ -906,8 +903,8 @@ class WebDriverElement(ElementAPI):
 
     def find_by_id(self, selector, wait_time=None):
         return self.find_by(
-            self._element.find_elements_by_id,
-            selector,
+            self._element.find_elements,
+            finder_kwargs={'by': By.ID, 'value': selector},
             original_find="id",
             wait_time=wait_time,
         )
@@ -918,24 +915,16 @@ class WebDriverElement(ElementAPI):
         )
 
     def scroll_to(self):
-        """
-        Scroll to the current element.
-        """
+        """Scroll to the current element."""
         self.driver.execute_script("arguments[0].scrollIntoView(true);", self._element)
 
     def mouse_over(self):
-        """
-        Performs a mouse over the element.
-
-        """
+        """Perform a mouse over the element."""
         self.scroll_to()
         ActionChains(self.driver).move_to_element(self._element).perform()
 
     def mouse_out(self):
-        """
-        Performs a mouse out the element.
-
-        """
+        """Perform a mouse out the element."""
         self.scroll_to()
         size = self._element.size
 
@@ -944,30 +933,25 @@ class WebDriverElement(ElementAPI):
             ActionChains(self.driver).move_to_element_with_offset(
                 self._element, -10, -10).click().perform()
         except MoveTargetOutOfBoundsException:
-            ActionChains(self.driver).move_to_element_with_offset(
-                self._element, size['width'] + 10, 10).click().perform()
+            try:
+                ActionChains(self.driver).move_to_element_with_offset(
+                    self._element, size['width'] + 10, 10).click().perform()
+            except MoveTargetOutOfBoundsException:
+                ActionChains(self.driver).move_to_element_with_offset(
+                    self._element, 10, size['height'] + 10).click().perform()
 
     def double_click(self):
-        """
-        Performs a double click in the element.
-
-        """
+        """Perform a double click in the element."""
         self.scroll_to()
         ActionChains(self.driver).double_click(self._element).perform()
 
     def right_click(self):
-        """
-        Performs a right click in the element.
-
-        """
+        """Perform a right click in the element."""
         self.scroll_to()
         ActionChains(self.driver).context_click(self._element).perform()
 
     def drag_and_drop(self, droppable):
-        """
-        Performs drag a element to another elmenet.
-
-        """
+        """Drag an element to another element."""
         self.scroll_to()
         ActionChains(self.driver).drag_and_drop(self._element, droppable._element).perform()
 
@@ -976,18 +960,20 @@ class WebDriverElement(ElementAPI):
         height = self.driver.execute_script("return Math.max(document.body.scrollHeight, document.body.offsetHeight);")
         self.driver.set_window_size(width, height)
 
-    def screenshot(self, name='', suffix='.png', full=False):
-        name = name or ''
+    def screenshot(self, name='', suffix='.png', full=False, unique_file=True):
 
-        (fd, filename) = tempfile.mkstemp(prefix=name, suffix=suffix)
-        # don't hold the file
-        os.close(fd)
+        filename = '{}{}'.format(name, suffix)
+
+        if unique_file:
+            (fd, filename) = tempfile.mkstemp(prefix=name, suffix=suffix)
+            # don't hold the file
+            os.close(fd)
 
         if full:
             ori_window_size = self.driver.get_window_size()
             self._full_screen()
 
-        target = self.screenshot_as_png()
+        self._element.screenshot(filename)
 
         if full:
             # Restore screen size
@@ -995,42 +981,9 @@ class WebDriverElement(ElementAPI):
             height = ori_window_size.get('height')
             self.driver.set_window_size(width, height)
 
-        target.save(filename)
-
         return filename
 
-    def screenshot_as_png(self):
-        try:
-            from PIL import Image
-        except ImportError:
-            raise NotImplementedError('Element screenshot need the Pillow dependency. '
-                                      'Please use "pip install Pillow" install it.')
-
-        full_screen_png = self.driver.get_screenshot_as_png()
-
-        full_screen_bytes = BytesIO(full_screen_png)
-
-        im = Image.open(full_screen_bytes)
-        im_width, im_height = im.size[0], im.size[1]
-        window_size = self.driver.get_window_size()
-        window_width = window_size['width']
-
-        ratio = im_width * 1.0 / window_width
-        height_ratio = im_height / ratio
-
-        im = im.resize((int(window_width), int(height_ratio)))
-
-        location = self._element.location
-        x, y = location['x'], location['y']
-
-        pic_size = self._element.size
-        w, h = pic_size['width'], pic_size['height']
-
-        box = x, y, x + w, y + h
-        box = [int(i) for i in box]
-        target = im.crop(box)
-
-        return target
-
-    def __getitem__(self, attr):
-        return self._element.get_attribute(attr)
+    @property
+    def shadow_root(self) -> ShadowRootElement:
+        shadow_root = self._element.shadow_root
+        return ShadowRootElement(shadow_root, self)
