@@ -4,9 +4,15 @@
 # Use of this source code is governed by a BSD-style
 # license that can be found in the LICENSE file.
 
-from msedge.selenium_tools import Edge, EdgeOptions
-from splinter.driver.webdriver import BaseWebDriver, WebDriverElement
-from splinter.driver.webdriver.cookie_manager import CookieManager
+# Selenium 3 compatibility
+try:
+    from msedge.selenium_tools import Edge
+    from msedge.selenium_tools import EdgeOptions as Options
+except ImportError:
+    from selenium.webdriver import Edge
+    from selenium.webdriver.edge.options import Options
+
+from splinter.driver.webdriver import BaseWebDriver
 
 
 class WebDriver(BaseWebDriver):
@@ -25,7 +31,7 @@ class WebDriver(BaseWebDriver):
         **kwargs
     ):
 
-        options = EdgeOptions() or options
+        options = Options() or options
 
         if user_agent is not None:
             options.add_argument("--user-agent=" + user_agent)
@@ -42,10 +48,6 @@ class WebDriver(BaseWebDriver):
 
         options.use_chromium = chromium
 
-        self.driver = Edge(options=options, **kwargs)
+        driver = Edge(options=options, **kwargs)
 
-        self.element_class = WebDriverElement
-
-        self._cookie_manager = CookieManager(self.driver)
-
-        super(WebDriver, self).__init__(wait_time)
+        super(WebDriver, self).__init__(driver, wait_time)
