@@ -45,7 +45,7 @@ class CookieManager(CookieManagerAPI):
     def delete_all(self):
         self.driver.cookies.clearAll()
 
-    def all(self, verbose=False):
+    def all(self, verbose=False):  # NOQA: A003
         cookies = {}
         for key, value in self.driver.cookies.items():
             cookies[key] = value
@@ -98,7 +98,7 @@ class ZopeTestBrowser(ElementPresentMixIn, DriverAPI):
     def reload(self):
         self._browser.reload()
 
-    def quit(self):
+    def quit(self):  # NOQA: A003
         pass
 
     @property
@@ -216,20 +216,6 @@ class ZopeTestBrowser(ElementPresentMixIn, DriverAPI):
             query=name,
         )
 
-    def find_link_by_text(self, text):
-        return self._find_links_by_xpath("//a[text()='%s']" % text)
-
-    def find_link_by_href(self, href):
-        return self._find_links_by_xpath("//a[@href='%s']" % href)
-
-    def find_link_by_partial_href(self, partial_href):
-        return self._find_links_by_xpath("//a[contains(@href, '%s')]" % partial_href)
-
-    def find_link_by_partial_text(self, partial_text):
-        return self._find_links_by_xpath(
-            "//a[contains(normalize-space(.), '%s')]" % partial_text
-        )
-
     def fill(self, name, value):
         self.find_by_name(name=name).first._control.value = value
 
@@ -365,7 +351,7 @@ class ZopeTestBrowserElement(ElementAPI):
         xpath_str = '.{}'.format(_concat_xpath_from_str(text))
         return self.find_by_xpath(xpath_str)
 
-    def find_by_id(self, id):
+    def find_by_id(self, id):  # NOQA: A002
         elements = self._element.cssselect("#%s" % id)
         return ElementList([self.__class__(element, self) for element in elements])
 
